@@ -34,26 +34,12 @@ UiDriver.registerEventHandler("C_FUN_IS_CLEAN", function(msg, data, prevReturn)
     return isCleanOrForced(changeGeneration);
 });
 
+UiDriver.registerEventHandler("C_FUN_GET_HISTORY_GENERATION", function(msg, data, prevReturn) {
+    return editor.getHistoryGeneration();
+});
+
 UiDriver.registerEventHandler("C_CMD_SET_LANGUAGE", function(msg, data, prevReturn) {
-    Languages.setLanguage(editor, data);
-});
-
-/*
-    Sets the language by finding the more appropriate one for the file name.
-    E.g. path/to/worker.js => JavaScript
-    If nothing can be inferred from the file name, it looks at the editor content to see if it
-    can guess something (eg. files starting with #! are probably shell scripts).
-*/
-UiDriver.registerEventHandler("C_FUN_SET_LANGUAGE_FROM_FILENAME", function(msg, data, prevReturn) {
-    var lang = Languages.languageByFileName(editor, data);
-    Languages.setLanguage(editor, lang);
-    return lang;
-});
-
-/* Returns the id of the current language, and its data */
-UiDriver.registerEventHandler("C_FUN_GET_CURRENT_LANGUAGE", function(msg, data, prevReturn) {
-    var langId = Languages.currentLanguage(editor);
-    return {id: langId, lang: Languages.languages[langId]};
+    editor.setOption('mode', data);
 });
 
 UiDriver.registerEventHandler("C_CMD_SET_INDENTATION_MODE", function(msg, data, prevReturn) {
@@ -281,7 +267,7 @@ function applyReusedGroups(replacement, groups){
         //takes care of non-consecutive group reuse tokens,
         //i.e. in "\1 \3" with no "\2", the "\3" is ignored 
         groupToReuse = groups[iReuseGroup];
-        replacement = replacement.replace(new RegExp("\\\\"+iReuseGroup), groupToReuse);
+        replacement = replacement.replace(new RegExp("\\\\"+iReuseGroup, "g"), groupToReuse);
     }
     var groupReuseRegex = /\\([1-9])/g;
     //take care of all non-matched group reuse tokens (replace with empty string)
@@ -383,6 +369,7 @@ UiDriver.registerEventHandler("C_FUN_GET_LANGUAGES", function(msg, data, prevRet
 });
 
 UiDriver.registerEventHandler("C_CMD_SET_THEME", function(msg, data, prevReturn) {
+    var link = undefined;
     if (data.path != "") {
         var stylesheet = $("link[href='" + data.path + "']");
         if (stylesheet.length > 0) {
@@ -390,11 +377,17 @@ UiDriver.registerEventHandler("C_CMD_SET_THEME", function(msg, data, prevReturn)
             stylesheet.appendTo('head');
         } else {
             // Add the stylesheet
-            addStylesheet(data.path);
+            link = addStylesheet(data.path);
         }
     }
 
-    editor.setOption("theme", data.name);
+    if (link === undefined) {
+        editor.setOption("theme", data.name);
+    } else {
+        link.onload = function () {
+            editor.setOption("theme", data.name);
+        }
+    }
 });
 
 UiDriver.registerEventHandler("C_CMD_SET_FONT", function (msg, data, prevReturn) {
@@ -544,6 +537,25 @@ UiDriver.registerEventHandler("C_CMD_TRIM_LEADING_SPACE", function(msg, data, pr
     editLines(function (x) { return x.replace(/^\s+/, ""); });
 });
 
+UiDriver.registerEventHandler("C_CMD_ENABLE_MATH", function(msg, data, prevReturn) {
+    require(['features/latex/latex'], function(math) {
+        if (data) {
+            math.enable(editor);
+        } else {
+            math.disable(editor);
+        }
+    });
+})
+
+UiDriver.registerEventHandler("C_FUN_IS_MATH_ENABLED", function(msg, data, prevReturn) {
+    if (!require.defined('features/latex/latex')) {
+        return false
+    }
+
+    var math = require('features/latex/latex');
+    return math.isEnabled();
+})
+
 var tabToSpaceCounter = 0;
 function tabToSpaceHelper(match, offset, tabSize) {
     /*
